@@ -107,18 +107,15 @@ static gchar *GdmRemoteGreeter = NULL;
 static gchar *GdmGtkModulesList = NULL;
 static gchar *GdmChooser = NULL;
 static gchar *GdmLogDir = NULL;
-static gchar *GdmDisplayInit = NULL;
+static gchar *GdmDisplayInitDir = NULL;
 static gchar *GdmPostLogin = NULL;
 static gchar *GdmPreSession = NULL;
 static gchar *GdmPostSession = NULL;
 static gchar *GdmFailsafeXserver = NULL;
 static gchar *GdmXKeepsCrashing = NULL;
 static gchar *GdmHalt = NULL;
-static gchar *GdmHaltReal = NULL;
 static gchar *GdmReboot = NULL;
-static gchar *GdmRebootReal = NULL;
 static gchar *GdmSuspend = NULL;
-static gchar *GdmSuspendReal = NULL;
 static gchar *GdmServAuthDir = NULL;
 static gchar *GdmMulticastAddr;
 static gchar *GdmUserAuthDir = NULL;
@@ -198,31 +195,29 @@ static gboolean GdmConsoleNotify = TRUE;
 
 /* Config options used by slave */
 /* ---------------------------- */
-static gchar *GdmInitDir;
-static gchar *GdmGtkRc;
-static gchar *GdmGtkThemesToAllow;
-static gchar *GdmInclude;
-static gchar *GdmExclude;
-static gchar *GdmDefaultFace;
-static gchar *GdmLocaleFile;
-static gchar *GdmLogo;
-static gchar *GdmChooserButtonLogo;
-static gchar *GdmWelcome;
-static gchar *GdmRemoteWelcome;
-static gchar *GdmBackgroundProgram;
-static gchar *GdmBackgroundImage;
-static gchar *GdmBackgroundColor;
-static gchar *GdmGraphicalTheme;
-static gchar *GdmInfoMsgFile;
-static gchar *GdmInfoMsgFont;
-static gchar *GdmHost;
-static gchar *GdmHostImageDir;
-static gchar *GdmHosts;
-static gchar *GdmGraphicalThemeColor;
-static gchar *GdmGraphicalThemeDir;
-static gchar *GdmGraphicalThemes;
-static gchar *GdmPreFetchProgram;
-static gchar *GdmUse24Clock;
+static gchar *GdmGtkThemesToAllow = NULL;
+static gchar *GdmInclude = NULL;
+static gchar *GdmExclude = NULL;
+static gchar *GdmDefaultFace = NULL;
+static gchar *GdmLocaleFile = NULL;
+static gchar *GdmLogo = NULL;
+static gchar *GdmChooserButtonLogo = NULL;
+static gchar *GdmWelcome = NULL;
+static gchar *GdmRemoteWelcome = NULL;
+static gchar *GdmBackgroundProgram = NULL;
+static gchar *GdmBackgroundImage = NULL;
+static gchar *GdmBackgroundColor = NULL;
+static gchar *GdmGraphicalTheme = NULL;
+static gchar *GdmInfoMsgFile = NULL;
+static gchar *GdmInfoMsgFont = NULL;
+static gchar *GdmHost = NULL;
+static gchar *GdmHostImageDir = NULL;
+static gchar *GdmHosts = NULL;
+static gchar *GdmGraphicalThemeColor = NULL;
+static gchar *GdmGraphicalThemeDir = NULL;
+static gchar *GdmGraphicalThemes = NULL;
+static gchar *GdmPreFetchProgram = NULL;
+static gchar *GdmUse24Clock = NULL;
 
 static gint GdmPositionX;
 static gint GdmPositionY;
@@ -439,7 +434,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_X_KEEPS_CRASHING, &GdmXKeepsCrashing, &string_type);
    gdm_config_add_hash (GDM_KEY_BASE_XSESSION, &GdmBaseXsession, &string_type);
    gdm_config_add_hash (GDM_KEY_REMOTE_GREETER, &GdmRemoteGreeter, &string_type);
-   gdm_config_add_hash (GDM_KEY_DISPLAY_INIT_DIR, &GdmInitDir, &string_type);
+   gdm_config_add_hash (GDM_KEY_DISPLAY_INIT_DIR, &GdmDisplayInitDir, &string_type);
    gdm_config_add_hash (GDM_KEY_AUTOMATIC_LOGIN, &GdmAutomaticLogin, &string_type);
    gdm_config_add_hash (GDM_KEY_GTK_MODULES_LIST, &GdmGtkModulesList, &string_type);
    gdm_config_add_hash (GDM_KEY_REBOOT, &GdmReboot, &string_type);
@@ -458,7 +453,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_MULTICAST_ADDR, &GdmMulticastAddr, &string_type);
    gdm_config_add_hash (GDM_KEY_USER, &GdmUser, &string_type);
    gdm_config_add_hash (GDM_KEY_GROUP, &GdmGroup, &string_type);
-   gdm_config_add_hash (GDM_KEY_GTKRC, &GdmGtkRc, &string_type);
+   gdm_config_add_hash (GDM_KEY_GTKRC, &GdmGtkRC, &string_type);
    gdm_config_add_hash (GDM_KEY_GTK_THEME, &GdmGtkTheme, &string_type);
    gdm_config_add_hash (GDM_KEY_TIMED_LOGIN, &GdmTimedLogin, &string_type);
    gdm_config_add_hash (GDM_KEY_WILLING, &GdmWilling, &string_type);
@@ -614,16 +609,16 @@ gdm_get_custom_config (struct stat *statbuf)
 }
 
 /**
- * gdm_get_display_custom_config_file
+ * gdm_get_per_display_custom_config_file
  *
  * Returns the per-display config file for a given display
  * This is always the custom config file name with the display
  * appended, and never gdm.conf.
  */
-gchar *
-gdm_get_display_custom_config_file (gchar *display)
+static gchar *
+gdm_get_per_display_custom_config_file (gchar *display)
 {
-  g_strdup_printf ("%s%s", custom_config_file, display);
+  return g_strdup_printf ("%s%s", custom_config_file, display);
 }
  
 /**
@@ -721,6 +716,82 @@ gdm_get_value_bool (char *key)
    return FALSE;
 }
 
+/**
+ * Note that some GUI configuration parameters are read by the daemon,
+ * and in order for them to work, it is necessary for the daemon to 
+ * access a few keys in a per-display fashion.  These access functions
+ * allow the daemon to access these keys properly.
+ */
+
+/**
+ * gdm_get_value_int_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gint gdm_get_value_int_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      gint val = atoi (perdispval);
+      g_free (perdispval);
+      return val;
+   } else {
+      return (gdm_get_value_int (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_bool_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gboolean gdm_get_value_bool_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      if (perdispval[0] == 'T' ||
+          perdispval[0] == 't' ||
+          perdispval[0] == 'Y' ||
+          perdispval[0] == 'y' ||
+          atoi (perdispval) != 0) {
+		g_free (perdispval);
+		return TRUE;
+       } else {
+		return FALSE;
+       }
+   } else {
+      return (gdm_get_value_bool (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_string_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.  Note that this value needs to be freed,
+ * unlike the non-per-display version.
+ */
+gchar * gdm_get_value_string_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      return perdispval;
+   } else {
+      return (g_strdup (gdm_get_value_string (key)));
+   }
+}
+
 /**
  * gdm_config_key_to_string_per_display
  *
@@ -736,19 +807,23 @@ gdm_get_value_bool (char *key)
 void
 gdm_config_key_to_string_per_display (gchar *display, gchar *key, gchar **retval)
 {
-   gchar *file = gdm_get_display_custom_config_file (display);
+   gchar *file;
    gchar **splitstr = g_strsplit (key, "/", 2);
    *retval = NULL;
 
    if (display == NULL)
       return;
 
+   file = gdm_get_per_display_custom_config_file (display);
+
    if (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
        strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
-       strcmp (ve_sure_string (key), GDM_KEY_PAM_STACK) == 0) {
+       is_key (key, GDM_KEY_PAM_STACK)) {
       gdm_config_key_to_string (file, key, retval);
    }
 
+   g_free (file);
+
    return;
 }
 
@@ -765,6 +840,7 @@ gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
    VeConfig *cfg = ve_config_get (file);
    GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
    gchar **splitstr = g_strsplit (key, "/", 2);
+   GList *list;
    *retval = NULL;
 
    /* Should not fail, all keys should have a category. */
@@ -775,7 +851,7 @@ gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
    if (cfg == NULL)
       return;
 
-   GList *list = ve_config_get_keys (cfg, splitstr[0]);
+   list = ve_config_get_keys (cfg, splitstr[0]);
    while (list != NULL) {
       gchar *display_key     = (char *)list->data;
       gchar *display_fullkey = g_strdup_printf ("%s/%s", splitstr[0], display_key);
@@ -1215,7 +1291,7 @@ gdm_set_value_bool (gchar *key, gboolean value)
    _gdm_set_value_bool (key, value, TRUE);
 }
 
-void
+static void
 _gdm_set_value_int (gchar *key, gint value, gboolean doing_update)
 {
    gint *setting     = gdm_config_hash_lookup (val_hash, key);
@@ -1558,7 +1634,7 @@ gdm_update_config (gchar* key)
    struct stat statbuf, custom_statbuf;
    VeConfig *cfg;
    VeConfig *custom_cfg = NULL;
-   gboolean rc;
+   gboolean rc = FALSE;
 
    /*
     * Do not allow these keys to be updated, since GDM would need
@@ -1618,7 +1694,7 @@ gdm_update_config (gchar* key)
       ve_config_destroy (cfg);
       if (custom_cfg != NULL)
          ve_config_destroy (custom_cfg);
-      return rc;
+      return TRUE;
    }
 
    type = gdm_config_hash_lookup (type_hash, key);
@@ -1805,7 +1881,7 @@ gdm_load_displays (VeConfig *cfg, GList *list )
  *
  * Called by gdm_config_parse in a loop to set each key.
  */
-void
+static void
 gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
 {
    gchar *key               = (gchar *)key_in;
